{----------------------------------------------------------------------------
|
| Module: CodeGen.Pas
|
| Description: Routines to create the code template for the ClassGen wizard.
|
| History:       , 1998. MB first version.
|          Oct 16, 2001. MB Changed for Spider ODB version 2
|
|---------------------------------------------------------------------------}
unit CodeGen;

interface

uses
    WizForm; { for TWizardForm }

function CreateUnit( const MainForm : TWizardForm;
                     const FileName : String ) : Boolean;

{--------------------------------------------------------------------------}

implementation

uses
    SysUtils, { for ExtractFileName }
    Controls, { for mrYes }
    Dialogs,  { for MessageDlg }
    ObjBuckt, { for TStringBucket }
    OdbField, { for TOdbField }
    OdbIndex, { for TOdbIndex }
    OdbMisc;  { for RightPad }

const
    CMargin = '    ';

{--------------------------------------------------------------------------}

function ExtractFileNameWithoutExt( const FullFileName : String) : String;
begin
    Result := ChangeFileExt(ExtractFileName(FullFileName), '');
end;

{--------------------------------------------------------------------------}

procedure WriteInterface( const MainForm : TWizardForm;
                          var   OutFile  : System.Text);
var
    Index : Integer;
begin
  with MainForm do
  begin
    WriteLn(OutFile, '{---------------------------------------------------------------------------');
    WriteLn(OutFile, '|');
    WriteLn(OutFile, '|  Module: ' + ExtractFileNameWithoutExt(FileNameEdit.Text));
    WriteLn(OutFile, '|');
    WriteLn(OutFile, '|  Description: ' + ClassNameEdit.Text + ' class and object-relational definition.');
    WriteLn(OutFile, '|');
    WriteLn(OutFile, '|  History: ' + DateToDateStr( SysUtils.Date, 'mmm dd, yyyy') + '. Created by ClassGen wizard.');
    WriteLn(OutFile, '|');
    WriteLn(OutFile, '|---------------------------------------------------------------------------}');
    WriteLn(OutFile, 'unit ' + ExtractFileNameWithoutExt(FileNameEdit.Text) + ';');
    WriteLn(OutFile);
    WriteLn(OutFile, 'interface');
    WriteLn(OutFile);
    WriteLn(OutFile, 'uses');
    WriteLn(OutFile, CMargin + 'ObjPerst, { for TObjectReader, TObjectWriter, CloneObj }');
    WriteLn(OutFile, CMargin + 'Odb;      { for TORMapping }');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'type');
    WriteLn(OutFile);
    WriteLn(OutFile, ClassNameEdit.Text + ' = class(' + AncestorClassEdit.Text + ')');
    WriteLn(OutFile, 'protected');

    for Index := 0 to DataMemberLB.Items.Count-1 do
        WriteLn(OutFile, CMargin + 'F' +
                         RightPad( MemberName(Index), ' ', 20) + ' : ' +
                         MemberTypeStr(Index) + ';');

    WriteLn(OutFile);

    for Index := 0 to DataMemberLB.Items.Count-1 do
        WriteLn(OutFile, CMargin + 'procedure Set' +
                         MemberName(Index) + '(const In' +
                         MemberName(Index) + ' : ' +
                         MemberTypeStr(Index) + ');');

    WriteLn(OutFile);
    WriteLn(OutFile, 'public');

    WriteLn(OutFile, CMargin + 'constructor Create;');
    WriteLn(OutFile, CMargin + 'destructor Destroy; override;');
    WriteLn(OutFile);

    for Index := 0 to DataMemberLB.Items.Count-1 do
    begin
        WriteLn(OutFile, CMargin + 'property ' +
                         MemberName(Index) + ' : ' +
                         MemberTypeStr(Index));
        WriteLn(OutFile, CMargin + CMargin +
                         ' read  F' + MemberName(Index));
        WriteLn(OutFile, CMargin + CMargin +
                         ' write Set' + MemberName(Index) + ';');
        WriteLn(OutFile);
    end;

    WriteLn(OutFile, 'end;');

    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);

    WriteLn(OutFile, 'type');
    WriteLn(OutFile);
    WriteLn(OutFile, ORMappingName + ' = class(TORMapping)');
    WriteLn(OutFile, CMargin + 'function GetObjectType : TClass; override;');
    WriteLn(OutFile, CMargin + 'procedure DefineFields; override;');
    WriteLn(OutFile, CMargin + 'procedure DefineIndices; override;');
    WriteLn(OutFile, CMargin + 'procedure UpdateFieldsFromObj(const Obj : TObject); override;');
    if (GetAutoIncField <> nil) then
        WriteLn(OutFile, CMargin + 'procedure UpdateObjFromFields(const Obj : TObject); override;');

    WriteLn(OutFile, 'end;');


    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure Write' + ClassNameWithoutT + '(');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Obj     : TObject;');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Writer  : TObjectWriter );');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure Read' + ClassNameWithoutT + '(');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Obj     : TObject;');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Reader  : TObjectReader );');

  end; { with MainForm }

end;

{--------------------------------------------------------------------------}

procedure WriteImplementation( const MainForm : TWizardForm;
                               var   OutFile  : System.Text);
var
    Index       : Integer;
    Field       : TOdbField;
    OdbIndex    : TOdbIndex;
    Str1        : String;
    Str2        : String;
    StringCombo : TStringCombo;
begin
  with MainForm do
  begin
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'implementation');
    WriteLn(OutFile);
    WriteLn(OutFile, 'uses');
    WriteLn(OutFile, CMargin + 'OdbIndex, { for iaXXXX }');
    WriteLn(OutFile, CMargin + 'OdbField; { for oftString }');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure Write' + ClassNameWithoutT + '(');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Obj     : TObject;');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Writer  : TObjectWriter );');
    WriteLn(OutFile, 'var');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' : ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile, 'begin');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' := Obj as ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile);

    for Index := 0 to DataMemberLB.Items.Count-1 do
    begin
        Write(OutFile, CMargin + 'Writer.Write');
        case MemberType(Index) of
            dmtBoolean :
            begin
                WriteLn(OutFile, 'Boolean(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;

            dmtByte,
            dmtInteger,
            dmtLongInt,
            dmtWord,
            dmtShortInt :
            begin
                WriteLn(OutFile, 'Integer(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;

            dmtComp,
            dmtDouble,
            dmtExtended,
            dmtSingle :
            begin
                WriteLn(OutFile, 'Float(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;

            dmtChar :
            begin
                WriteLn(OutFile, 'Char(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;

            dmtString :
            begin
                WriteLn(OutFile, 'String(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;

            dmtObject :
            begin
                WriteLn(OutFile, 'ChildObject(' + ClassNameWithoutT + '.F' + MemberName(Index) + ');');
            end;
        end; { case }
    end;

    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure Read' + ClassNameWithoutT + '(');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Obj     : TObject;');
    WriteLn(OutFile, CreateStr(' ', 8) + 'const Reader  : TObjectReader );');
    WriteLn(OutFile, 'var');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' : ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile, 'begin');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' := Obj as ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile);

    for Index := 0 to DataMemberLB.Items.Count-1 do
    begin
        Write(OutFile, CMargin + ClassNameWithoutT + '.F' +
                       MemberName(Index) + ' := Reader.Read');

        case MemberType(Index) of
            dmtBoolean :
            begin
                WriteLn(OutFile, 'Boolean;');
            end;

            dmtByte,
            dmtInteger,
            dmtLongInt,
            dmtWord,
            dmtShortInt :
            begin
                WriteLn(OutFile, 'Integer;');
            end;

            dmtComp,
            dmtDouble,
            dmtExtended,
            dmtSingle :
            begin
                WriteLn(OutFile, 'Float;');
            end;

            dmtChar :
            begin
                WriteLn(OutFile, 'Char;');
            end;

            dmtString :
            begin
                WriteLn(OutFile, 'String;');
            end;

            dmtObject :
            begin
                WriteLn(OutFile, 'ChildObject;');
            end;
        end; { case }

    end;

    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);

    WriteLn(OutFile, 'constructor ' + ClassNameEdit.Text + '.Create;');
    WriteLn(OutFile, 'begin');
    WriteLn(OutFile, CMargin + 'inherited Create;');

    for Index := 0 to DataMemberLB.Items.Count-1 do
    begin
        if MemberType(Index) = dmtObject then
            WriteLn(OutFile, CMargin + 'F' + MemberName(Index) + ' := ' +
                             MemberTypeStr(Index) + '.Create;');
    end;
    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);

    WriteLn(OutFile, 'destructor ' + ClassNameEdit.Text + '.Destroy;');
    WriteLn(OutFile, 'begin');
    for Index := DataMemberLB.Items.Count-1 downto 0 do
    begin
        if MemberType(Index) = dmtObject then
            WriteLn(OutFile, CMargin + 'F' + MemberName(Index) + '.Free;');
    end;
    WriteLn(OutFile, CMargin + 'inherited Destroy;');
    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);

    for Index := 0 to DataMemberLB.Items.Count-1 do
    begin
        WriteLn(OutFile, 'procedure ' + ClassNameEdit.Text + '.Set' +
                         MemberName(Index) + '(const In' +
                         MemberName(Index) + ' : ' +
                         MemberTypeStr(Index) + ');');
        WriteLn(OutFile, 'begin');
        if MemberType(Index) = dmtObject then
        begin
            WriteLn(OutFile, CMargin + 'F' + MemberName(Index) + '.Free;');
            WriteLn(OutFile, CMargin + 'F' + MemberName(Index) + ' := CloneObj(In' + MemberName(Index) + ');');
        end
        else
            WriteLn(OutFile, CMargin + 'F' + MemberName(Index) + ' := In' + MemberName(Index) + ';');
        WriteLn(OutFile, 'end;');
        WriteLn(OutFile);
    end;

    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'function ' + ORMappingName + '.GetObjectType : TClass;');
    WriteLn(OutFile, 'begin');
    WriteLn(OutFile, CMargin + 'Result := ' + ClassNameEdit.Text +';');
    WriteLn(OutFile, 'end;');

    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure ' + ORMappingName + '.UpdateFieldsFromObj( const Obj    : TObject );');
    WriteLn(OutFile, 'var');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' : ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile, 'begin');
    WriteLn(OutFile, CMargin + ClassNameWithoutT + ' := Obj as ' + ClassNameEdit.Text + ';');
    WriteLn(OutFile);
    for Index := 0 to FieldLB.Items.Count-1 do
    begin
        Field := GetField(Index);

        case Field.FieldType of
            oftInteger,
            oftAutoInc  : Str1 := 'AsInteger';
            oftString   : Str1 := 'AsString';
            oftFloat    : Str1 := 'AsFloat';
        end;

        WriteLn(OutFile, CMargin + 'FieldByName(''' + Field.Name + ''').' +
                         Str1 + ' := ' + ClassNameWithoutT + '.' + Field.Name + ';');

    end;

    WriteLn(OutFile, 'end;');
    Field := GetAutoIncField;
    if Field <> nil then
    begin
        WriteLn(OutFile);
        WriteLn(OutFile, '{--------------------------------------------------------------------------}');
        WriteLn(OutFile);
        WriteLn(OutFile, 'procedure ' + ORMappingName + '.UpdateObjFromFields( const Obj    : TObject );');
        WriteLn(OutFile, 'var');
        WriteLn(OutFile, CMargin + ClassNameWithoutT + ' : ' + ClassNameEdit.Text + ';');
        WriteLn(OutFile, 'begin');
        WriteLn(OutFile, CMargin + ClassNameWithoutT + ' := Obj as ' + ClassNameEdit.Text + ';');
        WriteLn(OutFile);
        WriteLn(OutFile, CMargin + ClassNameWithoutT + '.' + Field.Name + ' := ' +
                         'FieldByName(''' + Field.Name + ''').AsInteger;');
        WriteLn(OutFile, 'end;');
    end;
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure ' + ORMappingName + '.DefineFields;');
    WriteLn(OutFile, 'begin');
    for Index := 0 to FieldLB.Items.Count-1 do
    begin
        Field := GetField(Index);

        Str2 := '0';
        case Field.FieldType of
            oftInteger  : Str1 := 'oftInteger';
            oftAutoInc  : Str1 := 'oftAutoInc';
            oftString   :
            begin
                Str1 := 'oftString';
                Str2 := IntToStr(Field.Size);
            end;
            oftFloat    : Str1 := 'oftFloat';
        end;

        WriteLn(OutFile, CMargin + 'AddField(''' + Field.Name + ''', '  +
                         Str1 + ', ' + Str2 + ');');

    end;
    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);
    WriteLn(OutFile, 'procedure ' + ORMappingName + '.DefineIndices;');
    WriteLn(OutFile, 'begin');
    for Index := 0 to IndexLB.Items.Count-1 do
    begin
        OdbIndex := GetIndex(Index);

        Str1 := '';
        if iaPrimary in OdbIndex.Attributes then
            Str1 := Str1 + 'iaPrimary, ';

        if iaCaseInsensitive in OdbIndex.Attributes then
            Str1 := Str1 + 'iaCaseInsensitive, ';

        if iaDescending in OdbIndex.Attributes then
            Str1 := Str1 + 'iaDescending, ';

        if iaUnique in OdbIndex.Attributes then
            Str1 := Str1 + 'iaUnique, ';

        if Str1 <> '' then
            Delete( Str1, Length(Str1) - 1, 2 );

        Str2 := '';
        if OdbIndex.FieldNames.GotoFirst then
        repeat
            StringCombo := OdbIndex.FieldNames.CurrentObj as TStringCombo;

            Str2 := Str2 + '''' + StringCombo.Str + ''', ';
        until not OdbIndex.FieldNames.GotoNext;
        if Str2 <> '' then
            Delete( Str2, Length(Str2) - 1, 2 );

        WriteLn(OutFile, CMargin + 'AddIndex(''' + OdbIndex.Name + ''', ['  +
                         Str1 + '], [' + Str2 + ']);');

    end;
    WriteLn(OutFile, 'end;');
    WriteLn(OutFile);
    WriteLn(OutFile, '{--------------------------------------------------------------------------}');
    WriteLn(OutFile);

    WriteLn(OutFile, 'initialization');
    WriteLn(OutFile);
    WriteLn(OutFile, 'RegisterStreamable(');
    WriteLn(OutFile, CMargin + CMargin + ClassNameEdit.Text +',');
    WriteLn(OutFile, CMargin + CMargin + '[Read' + ClassNameWithoutT + '],');
    WriteLn(OutFile, CMargin + CMargin + '[Write' + ClassNameWithoutT + ']');
    WriteLn(OutFile, CMargin + CMargin + ');');
    WriteLn(OutFile);
    WriteLn(OutFile, 'end.');
  end; { with MainForm }
end;

{--------------------------------------------------------------------------}

function CreateUnit( const MainForm : TWizardForm;
                     const FileName : String ) : Boolean;
var
   OutFile : System.Text;
begin
    Result := False;

    if FileExists(MainForm.FileNameEdit.Text) then
    begin
        if MessageDlg( 'File ' + MainForm.FileNameEdit.Text + ' exists' + #13 +
                       'Overwrite ?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
            Exit;
    end;

    AssignFile( OutFile, MainForm.FileNameEdit.Text);
    Rewrite(OutFile);
    try
        WriteInterface(MainForm, OutFile);
        WriteImplementation(MainForm, OutFile);

        MessageDlg( 'Unit ' + MainForm.FileNameEdit.Text + ' created successfully.',
                    mtInformation, [mbOk], 0);

        Result := True;
    finally
        CloseFile(OutFile);
    end;
end;

{--------------------------------------------------------------------------}

end.
